@@ -1,34 +1,26 @@
-import json
 import os
 
 import docker
-from dagster import check
-from dagster.core.launcher.base import LaunchRunContext, RunLauncher
-from dagster.core.storage.tags import DOCKER_IMAGE_TAG
-from dagster.grpc.types import ExecuteRunArgs
-from dagster.serdes import ConfigurableClass, serialize_dagster_namedtuple
 from dagster_docker.utils import DOCKER_CONFIG_SCHEMA, validate_docker_config, validate_docker_image
 
+from dagster import check
+from dagster.core.launcher.base import (
+    CheckRunHealthResult,
+    LaunchRunContext,
+    ResumeRunContext,
+    RunLauncher,
+    WorkerStatus,
+)
+from dagster.core.storage.pipeline_run import PipelineRun
+from dagster.core.storage.tags import DOCKER_IMAGE_TAG
+from dagster.grpc.types import ExecuteRunArgs, ResumeRunArgs
+from dagster.serdes import ConfigurableClass
+
 DOCKER_CONTAINER_ID_TAG = "docker/container_id"
 
 
 class DockerRunLauncher(RunLauncher, ConfigurableClass):
-    """Launches runs in a Docker container.
-
-    Args:
-        image (Optional[str]): The docker image to be used if the repository does not specify one.
-        registry (Optional[Dict[str, str]]): Information for using a non-local docker registry.
-            If set, should include ``url``, ``username``, and ``password`` keys.
-        env_vars (Optional[List[str]]): The list of environment variables names to forward to the
-            docker container.
-        network (Optional[str]): Name of the network this container to which to connect the
-            launched container at creation time. DEPRECATED, prefer networks
-        networks (Optional[List[str]]): List of networks to which to connect the
-            launched container at creation time.
-        container_kwargs(Optional[Dict[str, Any]]): Additional kwargs to pass into
-            containers.create. See https://docker-py.readthedocs.io/en/stable/containers.html
-            for the full list of available options.
-    """
+    """Launches runs in a Docker container."""
 
     def __init__(
         self,
@@ -82,12 +74,7 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
             )
         return client
 
-    def launch_run(self, context: LaunchRunContext) -> None:
-
-        run = context.pipeline_run
-
-        pipeline_code_origin = context.pipeline_code_origin
-
+    def _get_docker_image(self, pipeline_code_origin):
         docker_image = pipeline_code_origin.repository_origin.container_image
 
         if not docker_image:
@@ -97,17 +84,9 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
             raise Exception("No docker image specified by the instance config or repository")
 
         validate_docker_image(docker_image)
+        return docker_image
 
-        input_json = serialize_dagster_namedtuple(
-            ExecuteRunArgs(
-                pipeline_origin=pipeline_code_origin,
-                pipeline_run_id=run.run_id,
-                instance_ref=self._instance.get_ref(),
-            )
-        )
-
-        command = "dagster api execute_run {}".format(json.dumps(input_json))
-
+    def _launch_container_with_command(self, run, docker_image, command):
         docker_env = (
             {env_name: os.getenv(env_name) for env_name in self.env_vars} if self.env_vars else {}
         )
@@ -156,6 +135,36 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
 
         container.start()
 
+    def launch_run(self, context: LaunchRunContext) -> None:
+        run = context.pipeline_run
+        pipeline_code_origin = context.pipeline_code_origin
+        docker_image = self._get_docker_image(pipeline_code_origin)
+
+        command = ExecuteRunArgs(
+            pipeline_origin=pipeline_code_origin,
+            pipeline_run_id=run.run_id,
+            instance_ref=self._instance.get_ref(),
+        ).get_command_args()
+
+        self._launch_container_with_command(run, docker_image, command)
+
+    @property
+    def supports_resume_run(self):
+        return True
+
+    def resume_run(self, context: ResumeRunContext) -> None:
+        run = context.pipeline_run
+        pipeline_code_origin = context.pipeline_code_origin
+        docker_image = self._get_docker_image(pipeline_code_origin)
+
+        command = ResumeRunArgs(
+            pipeline_origin=pipeline_code_origin,
+            pipeline_run_id=run.run_id,
+            instance_ref=self._instance.get_ref(),
+        ).get_command_args()
+
+        self._launch_container_with_command(run, docker_image, command)
+
     def _get_container(self, run):
         if not run or run.is_finished:
             return None
@@ -167,7 +176,7 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
 
         try:
             return self._get_client().containers.get(container_id)
-        except Exception:  # pylint: disable=broad-except
+        except Exception:
             return None
 
     def can_terminate(self, run_id):
@@ -191,3 +200,17 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
         container.stop()
 
         return True
+
+    @property
+    def supports_check_run_worker_health(self):
+        return True
+
+    def check_run_worker_health(self, run: PipelineRun):
+        container = self._get_container(run)
+        if container == None:
+            return CheckRunHealthResult(WorkerStatus.NOT_FOUND)
+        if container.status == "running":
+            return CheckRunHealthResult(WorkerStatus.RUNNING)
+        return CheckRunHealthResult(
+            WorkerStatus.FAILED, msg=f"Container status is {container.status}"
+        )
